/**
 * Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

'use strict';

const transitiveNPM3Dep = require('npm3-transitive-dep');

module.exports = () => ({
  isUnmocked: () => true,
  transitiveNPM3Dep: transitiveNPM3Dep(),
  internalImplementation: transitiveNPM3Dep.internalImplementation,
});
