/**
 * Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
Object.assign(exports, {getMain, requireFromMain});

function getMain() {
  return require.main;
}

function requireFromMain(pkg) {
  return getMain().require(pkg);
}
