/**
 * Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
// @ts-expect-error
// eslint-disable-next-line @typescript-eslint/no-unused-vars
interface NotUsedButTakesUpLines {
  a: number;
  b: string;
}

export function error() {
  throw new Error('This did not work!');
}
