/**
 * Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
'use strict';

test('works just fine', () => {
  console.log('This is a log message.');

  console.info('This is an info message.');

  console.warn('This is a warning message.');

  console.error('This is an error message.');
});
