/**
 * Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * @jest-environment ./CircusHandleTestEventEnvironment.js
 */

beforeEach(() => {});

test('test name here', () => {
  expect(true).toBe(true);
});

test('second test name here', () => {
  expect(true).toBe(true);
});
