/**
 * Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

// https://jonasjacek.github.io/colors/

export const aForeground2 = 90;
export const aBackground2 = 225;

export const bForeground2 = 23;
export const bBackground2 = 195;

export type RGB = [number, number, number];

export const aForeground3: RGB = [0x80, 0, 0x80];
export const aBackground3: RGB = [0xff, 0xd7, 0xff];

export const bForeground3: RGB = [0, 0x5f, 0x5f];
export const bBackground3: RGB = [0xd7, 0xff, 0xff];
