/**
 * Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 */

// eslint-disable-next-line no-unused-vars
const createPlugin = require('../utils').createPlugin;

// We inject the call to "createPlugin('bar') through the transformer"
