/**
 * Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
'use strict';

describe('todos in verbose reporter', () => {
  test.todo('this');
  test.todo('should');
  test.todo('work');
});
